@@ -1,3 +1,3 @@
 """fyda - the interface for your data"""
 from .base import DataBank, ProjectConfig
-from .base import load, load_s3, data_path
+from .base import load, load_s3, data_path, dir_path
