@@ -1,5 +1,4 @@
 """Base module for fyda."""
-import importlib
 import json
 import os
 import pickle
@@ -19,6 +18,10 @@ from .errorhandling import NoShortcutError
 # ?? Option values for behavior with duplicates. (Overwrite/keep/rename)
 # Sanity checks for file assignment in .fydarc. Possibly get flexible there.
 # Future idea: some way to search through files/shortcuts; like fuzzy search
+# Integrate cloud-based file loading directly into load(). i.e. point ``root``
+#   to a bucket, and have fyda work its magic from there.
+# Additional keyword arguments in ``load`` and ``withdraw``, like sheet_name.
+# Better path handling in .fydarc. e.g. when quotation marks appear in the path
 
 
 # -----------------------------------------------------------------------------
@@ -72,28 +75,9 @@ class DataBank:
     root : str
         Path to root data folder. If none is provided, uses the default from
         ``.fydarc`` given by the ``conf_path`` parameter.
-
-    Attributes
-    ----------
-    shortcuts : dict
-        A mapping of shortcut values to their respective file objects.
-    tree : dict
-        The file tree representation of the ``root`` folder. Each folder can be
-        accessed as successive ``dict`` keys, e.g.
-        tree['level`']['level2'][...].
-    readers : dict
-        Maps between shortcuts and the required data reader for that file.
-
-    Methods
-    -------
-    :meth:`DataBank.deposit`
-    :meth:`DataBank.determine_shortcut`
-    :meth:`DataBank.rebase_shortcuts`
-    :meth:`DataBank.root_to_dict`
-    :meth:`DataBank.withdraw`
     """
 
-    def __init__(self, root=None):
+    def __init__(self, root=None, error='ignore'):
 
         if root is None:
             pc = ProjectConfig()
@@ -107,21 +91,24 @@ class DataBank:
         self._data = {}
         self._reader_map = {}
         self._forbid = {}
-        self._tree = self.root_to_dict(self._root)
+        self._tree = self.root_to_dict(self._root, error=error)
         # TODO rcusers information to avoid overwriting values set in config
 
     # We access attributes this way because dict is mutable
     # TODO: any way to warn people when they try to change these?
     @property
     def tree(self):
+        """Full tree of data root directory in python dictionary form."""
         return self.root_to_dict(self._root)
 
     @property
     def shortcuts(self):
+        """Mapping of shortcuts to absolute paths."""
         return self._data.copy()
 
     @property
     def readers(self):
+        """Mapping of shortcuts to their respective readers."""
         return self._reader_map.copy()
 
     def _determine_path(self, input_string):
@@ -194,7 +181,7 @@ class DataBank:
             reader = _pick_reader(filepath, error=error)
 
         # Update user list
-        default = default_shortcut(filepath)
+        default = _default_shortcut(filepath)
         # TODO make this better?
         if default in self._forbid:
             new_userlist = self._forbid[default]['in_use'] + [shortcut]
@@ -229,7 +216,7 @@ class DataBank:
 
         """
         # Base name without extension
-        default = default_shortcut(filepath)
+        default = _default_shortcut(filepath)
 
         if default not in self._forbid:
             return default, False
@@ -246,7 +233,7 @@ class DataBank:
     def encoding_level(self, fileref):
         """Get the encoding level for given file reference."""
 
-        default = default_shortcut(fileref)
+        default = _default_shortcut(fileref)
 
         if default in self._forbid:
             return self._forbid[default]['encode_level']
@@ -269,7 +256,7 @@ class DataBank:
                           'DataBank. Killing process.'.format(filepath))
             return
 
-        default = default_shortcut(filepath)
+        default = _default_shortcut(filepath)
         encode_level = self._forbid[default]['encode_level']
         users = self._forbid[default]['in_use']
         shortcut = _encode_shortcut(filepath, encode_level)
@@ -294,7 +281,7 @@ class DataBank:
 
         self._forbid[default]['in_use'] = users
 
-    def root_to_dict(self, root, auto_deposit=True):
+    def root_to_dict(self, root, auto_deposit=True, error='raise'):
         """
         Recursively convert root folder to native Python dictionary.
 
@@ -305,11 +292,14 @@ class DataBank:
         auto_deposit : bool
             If True, automatically call :meth:`DataBank.deposit` on any files
             found through the recursion to the ``shortcuts`` dict.
+        error : str, {'raise', 'ignore'}
+            Whether to ignore filetype errors or raise a
+            ``NotImplementedError``.
 
         Notes
         -----
-        Modified from `this`<https://btmiller.com/2015/03/17/represent-file
-        -structure-as-yaml-with-python.html>_ example by Blake Miller.
+        Modified from `this`<https://btmiller.com/2015/03/17/represent-file-structure-as-yaml-with-python.html>_
+        example by Blake Miller.
 
         """
 
@@ -324,16 +314,16 @@ class DataBank:
             # If it's a file, set "basename": "abspath to file"
             for f in filenames:
                 filepath = os.path.join(root, f)
-                directory[dn].update({default_shortcut(f): f})
+                directory[dn].update({_default_shortcut(f): f})
 
                 if auto_deposit:
-                    self.deposit(filepath)
+                    self.deposit(filepath, error=error)
 
             # If it's a directory, go down a level and start over
             if dirnames:
                 for d in dirnames:
                     directory[dn].update(
-                        self.root_to_dict(os.path.join(root, d)))
+                        self.root_to_dict(os.path.join(root, d), error=error))
 
             break  # We break here to stop the os.walk from doubling back
 
@@ -409,6 +399,12 @@ def _decode(reader, filename):
             return reader(fileobj)
 
 
+def _default_shortcut(filepath):
+    """Get the default shortcut name for a file."""
+
+    return os.path.splitext(os.path.basename(filepath))[0]
+
+
 def _encode_shortcut(filepath, encoding_level=0):
     """Get shortcut from filepath at given encoding level. 0 = base,
     1 = base.ext, 2 = folder/base.ext, 3 = folder_up/folder/base.ext,
@@ -420,7 +416,7 @@ def _encode_shortcut(filepath, encoding_level=0):
         raise ValueError("Encoding level for shortcut must be a positive "
                          "integer.")
     elif encoding_level == 0:
-        return default_shortcut(filepath)
+        return _default_shortcut(filepath)
 
     shortcut = os.path.basename(filepath)
     upstream = os.path.dirname(filepath)
@@ -498,7 +494,7 @@ def data_path(shortcut, root=None):
     db = DataBank(root)
 
     if shortcut in db.shortcuts:
-        return db.shortcuts[shortcut]
+        return os.path.abspath(db.shortcuts[shortcut])
     else:
         try:
             pc = ProjectConfig()
@@ -508,10 +504,31 @@ def data_path(shortcut, root=None):
             raise NoShortcutError(shortcut)
 
 
-def default_shortcut(filepath):
-    """Get the default shortcut name for a file."""
+def dir_path(shortcut, root=None):
+    """
+    Return absolute path to the directory referenced by ``shortcut``.
 
-    return os.path.splitext(os.path.basename(filepath))[0]
+    Parameters
+    ----------
+    shortcut : str
+        Shortcut reference for the folder/directory.
+    root : str
+        Root directory to use with :class:`DataBank`.
+
+    Returns
+    -------
+    path : str
+        Absolute path to directory.
+    """
+
+    db = DataBank(root)
+    pc = ProjectConfig()
+
+    path = pc['directories'][shortcut]
+    if path[0] == '~':
+        return os.path.abspath(os.path.expanduser(path))
+
+    return os.path.abspath(os.path.join(db._root, path))
 
 
 def load(file_name):
